#include <hi_mpi_avs_lut_generate.h>
#include <ot_common_avs.h>
#include <ot_common_avs_lut_generate.h>
#include <fstream>
#include <vector>
#include <iostream>

static td_phys_addr_t  g_avs_lut_addr = 0; 
static td_phys_addr_t  g_avs_mask_addr = 0; 
static td_char g_cal_file[OT_AVS_CALIBRATION_FILE_LEN]; 
static td_u32 g_width = 2688;
static td_u32 g_height = 1520;

#define FILE_MAX_LEN 256
#define MAX_CAMERA_NUM 8

typedef struct fine_package{
    int cameraIndex;
    int yaw;
    int pitch;
    int roll;
    int offsetX;
    int offsetY;
}FINE_PACKAGE, *LPFINE_PACKAGE;

class AvsFineTuningTool {
public:
    static AvsFineTuningTool* getInstance() {
        if (instance == nullptr) {
            instance = new AvsFineTuningTool(4);
        }
        return instance;
    }

    // set the yaw pitch roll that is concerned with the camera
    void SetYaw(int index, int yaw){m_finePackages[index].yaw = yaw;}
    void SetPitch(int index,int pitch){m_finePackages[index].pitch = pitch;}
    void SetRoll(int index,int roll){m_finePackages[index].roll = roll;}
    void SetOffsetX(int index,int offsetX){m_finePackages[index].offsetX = offsetX;}
    void SetOffsetY(int index,int offsetY){m_finePackages[index].offsetY = offsetY;}

    void setStitchingDistance(float distance) {m_StitchingDistance = distance;}

    void setAVSPLUTSavePath(std::string path){m_SaveAVSPPath = path;}

    // get the yaw pitch roll that is concerned with the camera
    int GetYaw(int index){return m_finePackages[index].yaw;}
    int GetPitch(int index){return m_finePackages[index].pitch;}
    int GetRoll(int index){return m_finePackages[index].roll;}
    int GetOffsetX(int index){return m_finePackages[index].offsetX;}
    int GetOffsetY(int index){return m_finePackages[index].offsetY;}

    // generate the LUT file
    void GenerateLutFile(int index);

    // Delete copy constructor and assignment operator to prevent copies
    AvsFineTuningTool(const AvsFineTuningTool&) = delete;
    AvsFineTuningTool& operator=(const AvsFineTuningTool&) = delete;

    static hi_u32 getCalFileLong(const hi_char* calFile) {
        FILE* fp;
        hi_u32 fileLen = 0;
        hi_s32 ret = 0;

        fp = fopen(calFile, "rb");
        if (fp == NULL) {
            printf("open file %s failed\n", calFile);
            return 0;
        }
        ret = fseek(fp, 0, SEEK_END);
        if (ret != 0) {
            printf("fseek failed\n");
            fclose(fp);
            return 0;
        }

        fileLen = ftell(fp);

        fclose(fp);
        
        return fileLen;
    }

    static hi_s32 readCalFile(const hi_char* calFile, hi_char* addr,hi_u32 size) {
        FILE* fp;
        hi_s32 readByte = 0;

        fp = fopen(calFile, "rb");
        if(fp == NULL) {
            printf("open file %s failed\n", calFile);
            return HI_FAILURE;
        }

        readByte = fread(addr, 1, size, fp);
        if(readByte != size) {
            printf("read file failed\n");
            fclose(fp);
            return HI_FAILURE;
        }

        fclose(fp);

        return HI_SUCCESS;

    }

    static hi_s32 writeLutFile(int index,std::string path,td_u32 length,td_u64 lut_output_addr[])
    {
        td_s32 i;
        td_char out_lut[MAX_CAMERA_NUM][FILE_MAX_LEN];
        FILE *lut_file = NULL;

        for(i = 0 ; i < MAX_CAMERA_NUM; i++) {
            if(i != index)
                continue; 
            std::string curPath;
            snprintf(out_lut[i], FILE_MAX_LEN, "avsp_mesh_out_%d.bin", i);
            curPath = path + out_lut[i];
            if(lut_output_addr[i] == 0) {
                continue;
            }
            lut_file = fopen(curPath.c_str(), "wb");
            if(lut_file == NULL) {
                return HI_FAILURE;
            }
            fwrite((td_char*)(td_uintptr_t)lut_output_addr[i], length, 1, lut_file);
            fclose(lut_file);
        }
    }

    // Add your fine-tuning methods here
private:
    AvsFineTuningTool(int camera) {
        m_cameraNumber = camera;
        m_finePackages.resize(camera);
    }

    ~AvsFineTuningTool() {
        // Clean up your tool here
    }

    // free the resources
    void FreeResources();

private:
    static AvsFineTuningTool* instance;
    std::vector<FINE_PACKAGE> m_finePackages;
    ot_avs_fine_tuning  m_fine_tuning_cfg;
    int m_cameraNumber;

    float m_StitchingDistance{2};

    // generate resouces
    td_void* m_avsMask;
    td_void* m_avsLut;

    // save path 
    std::string m_SaveAVSPPath;

};

