#ifndef __AVS_UPDATE_TUNING_HPP__
#define __AVS_UPDATE_TUNING_HPP__

#include <hi_mpi_avs_lut_generate.h>
#include <ot_common_avs.h>
#include <ot_common_avs_lut_generate.h>
#include <hi_common_avs.h>
#include <hi_mpi_avs.h>
#include <hi_common.h>
#include <hi_mpi_sys.h>
#include <string>
#include <stdexcept>

#define AVSP_LUT_NAME "avsp_mesh_out_"

class AvsUpdateTuning {
public:
    AvsUpdateTuning(hi_avs_grp index) {
        m_avs_grp_index = index;

        hi_s32 ret = 0;

        ret = hi_mpi_avs_get_grp_attr(m_avs_grp_index, &m_avs_grp_attr);

        if(ret != HI_SUCCESS) {
            printf("hi_mpi_avs_set_grp_attr failed\n");
            throw std::runtime_error("hi_mpi_avs_set_grp_attr failed");
        }

        m_avs_pipe_num = m_avs_grp_attr.pipe_num;

        ret = hi_mpi_sys_mmz_alloc(&m_avs_lut_addr, &m_avsLut, "avs_lut", NULL, OT_AVS_LUT_SIZE * sizeof(char) * m_avs_pipe_num);

        if (ret != HI_SUCCESS) {
            printf("hi_mpi_sys_mmz_alloc failed\n");
            throw std::runtime_error("hi_mpi_sys_mmz_alloc failed");
        }

        for(int i = 0; i < m_avs_pipe_num; i++) {
            m_LutArray[i] = (hi_u64)(hi_uintptr_t)m_avsLut + i * OT_AVS_LUT_SIZE;
        }

    }

    void setPath(std::string path) {
        m_strPath = path;
    }

    void readLutFile(int index,std::string path, hi_u32 length, hi_u64 lut_output_addr[]);

    hi_s32 updateGrpAttr();

private:
    hi_avs_grp m_avs_grp_index;
    hi_avs_grp_attr m_avs_grp_attr;

    hi_u64 m_LutArray[OT_AVS_MAX_CAMERA_NUM];

    std::string m_strPath;

    hi_u32 m_avs_pipe_num;

    hi_phys_addr_t m_avs_lut_addr;
    hi_void* m_avsLut;

};

#endif // __AVS_UPDATE_TUNING_HPP__